ABMaterial (ABM) Custom Component Canvas Knob
v1.0.0 (Build (20171122)

[b]Objectives[/b]
Create an ABM custom component for a knob control. 
There are two examples:
[i]TwoKnobs[/i]
The example application has two knobs Volume and Bass, which are turned using the mouse.
[img]http://www.rwblinn.de/b4j/b4jhowto/examples/b4jhowtoabmcustomcanvasknob-twoknobs.png[/img]

[i]Clock[/i]
A dynamic clock using 3 knobs which are updated.
[img]http://www.rwblinn.de/b4j/b4jhowto/examples/b4jhowtoabmcustomcanvasknob-clock.png[/img]

[b]Notes[/b]

* The ABM custom component is wrapped from the [url=https://github.com/aterrien/jQuery-Knob]jQuery-Knob[/url] project. Many thanks to the author.
* The ABM sample project is part of the B4JHowTos and is called B4JHowToABMCustomCanvasKnob.
* A comprehensive tutorial on how to create an ABM Custom Component can be found [url=http://make.rwblinn.de/viewtopic.php?f=52&t=197&p=197#p197]here[/url].
* Not the full code is shared in the description below.
* Software versions used B4J v5.90, ABM 4.00 (Dragonfly).

[b]Download[/b]
[url=http://www.rwblinn.de/b4j/b4jhowto/examples/b4jhowtoabmcustomcanvasknob.zip]B4J Source Code[/url].
The archive does not contain the full ABM folder structure, but only what is required for the examples (in folder custom/js).

To test the application, copy first the www folder (and only that folder) from the ABM Template folder.

[b]Build Steps[/b]
Below is outlined how to create an ABM application with a custom canvas knob.
Recommend to read the source of the previous mentioned examples. Start with the Two Knobs example.

[b]Prepare[/b]
Create a folder <path>\B4JHowToABMCustomCanvasKnob a new ABM project CustomCanvasKnobEx.
Copy the ABM Template folder to folder <path>\B4JHowToABMCustomCanvasKnob.
Rename abmtemplate.b4j and abmtemplate.b4j.meta to CustomCanvasKnobEx.b4j and CustomCanvasKnobEx.b4j.meta.
Make the initial changes to the classes according ABM guidance ( see ABM Demo Project ).
Changed to port 51050 in CustomCanvasKnobEx.b4j Process_Globals. The application URL is http://localhost:51050/CustomCanvasKnobEx.

[b]JavaScript jQuery-Knob Master[/b]
Download the master from [url=https://github.com/aterrien/jQuery-Knob]jQuery-Knob]GitHub[/url].
Unpack the zip and copy from the folder dist, the file jquery.knob.min.js to folder <path>\B4JHowToABMCustomCanvasKnob\Objects\www\js\custom\
Review the documentation (on GitHub) to understand how to use & configure the knob.
Befor starting to develop the application, created a HTML demo (see folder HTMLTests fullexample or simpleexample).

[b]Create the custom component class CustomCanvasKnob[/b]
Lookup the documented source code.
[i]Notes[/i]
To note is the usage of a map to set the knob properties.
This provides flexibility in settings properties. Important is to check if a property is set and set a default value if is null.

[i]Hints Knob Layout[/i]
The full example shows the various configuration possibilities.
To configure, define the HTML code in Sub ABMComp_Build and the JS code in Sub ABMComp_FirstRun.
Here two example - as mentioned look at the source of the fullexample.html.

#Example Knob size 100px
Sub ABMComp_Build
	Return $"<div id="${internalID}">
			 	<input id="${ABMComp.ID}i" type="text" value=${mKnobValue} data-linecap=round data-width="80%" class="${internalID}">
			 </div>"$
Sub ABMComp_FirstRun
	Dim script As String = $"
		$('.${internalID}').knob({
			"min":${mKnobMin},
			"max":${mKnobMax},
			"height":${mKnobHeight},
			"width":${mKnobWidth},
			"fgColor":"${mKnobfgColor}",
			"skin":"tron",
			"displayInput":${mKnobDisplayInput},
			'change': function (v) {
				    	b4j_raiseEvent('${ABMComp.ID}_changed', {'value':v});
			    	}
		}); 
    	b4j_raiseEvent('${ABMComp.ID}_changed', {'value':${mKnobValue}});
	"$

#Example responsive Knob with round linecaps
Note: The size of the knob is adjusted when the screen size changes. 
Sub ABMComp_Build
	Return $"<div id="${internalID}" style="width: 30%; border: 3px dashed; margin-bottom: 20px">
			 	<input id="${ABMComp.ID}i" type="text" value=${mKnobValue} data-linecap=round data-width="50%" class="${internalID}">
			 </div>"$
	OR 
	Return $"<div id="${internalID}">
			 	<input id="${ABMComp.ID}i" type="text" value=${mKnobValue} data-linecap=round data-width="80%" class="${internalID}">
			 </div>"$
Sub ABMComp_FirstRun
	Dim script As String = $"//Init the custom knob
		$('.${internalID}').knob({
			"min":${mKnobMin},
			"max":${mKnobMax},
			"fgColor":"${mKnobfgColor}",
			"skin":"tron",
			"displayInput":${mKnobDisplayInput},
			'change': function (v) {
				    	b4j_raiseEvent('${ABMComp.ID}_changed', {'value':v});
			    	}
		}); 
    	b4j_raiseEvent('${ABMComp.ID}_changed', {'value':${mKnobValue}});
	"$

[b]Create the page CustomCanvasKnobPage[/b]
This page uses the custom component CustomCanvasKnob.
Lookup the documented source code.

[i]Notes[/i]
The extra JS file as previous copied to the custom folder.
[code]
	page.AddExtraJavaScriptFile("custom/jquery.knob.min.js")
[/code]

Handling knob changes
Handle changes of the knob value which is raised by the b4j_raiseEvent('${internalID}_changed', {'value':m{mKnobValue}); defined in the JS code in CustomCanvasKnob class Sub ABMComp_FirstRun.
There are two knobs defined, each with its own changed event ${internalID}_changed.
[code]
	If Not(Params.IsInitialized) Then Return
	labelVolumeValue.Text = Round(Params.Get("value"))
[/code]

[b]Amend main Class CustomCanvasKnobEx.b4j[/b]
Set the port and add the CustomCanvasKnobPage.
[code]
[i]#Sub Process_Globals[/i]
Dim port As Int = 51050

[i]#Sub AppStart[/i]
Dim myPage As CustomCanvasKnobPage
myPage.Initialize		
myApp.AddPage(myPage.page)
[/code]

[b]Run[/b]
Run the application, access via browser http://localhost:51050/CustomCanvasKnobEx.
Turn the knob using the mouse. The display input label gets updated.
