﻿Type=Class
Version=5.9
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
'Class module ABMaterial CustomCanvasKnob
'@author Robert W.B. Linn @rwblinn.de
'@version 20171119
Sub Class_Globals
	Dim ABMComp As ABMCustomComponent
	Private mKnobValue As Int
	Private mKnobMin As Int
	Private mKnobMax As Int
	Private mKnobHeight As Int
	Private mKnobWidth As Int
	Private mKnobfgColor As String 
	Private mKnobDisplayInput As Boolean
End Sub

'Initializes the object.
'InternalPage - ABM page
'InternalID - Unique ID for the component
'Properties - Map holding the properties which are assigned to global vars used for creating the knob in ABMComp_FirstRun
'Not all possible properties are used - see https://github.com/aterrien/jQuery-Knob.
'Example Properties Map: <code>
'Dim m As Map = CreateMap("value":69, "fgcolor":"#0000FF")
'</code>
Public Sub Initialize(InternalPage As ABMPage, ID As String, Properties As Map)
	ABMComp.Initialize("ABMComp", Me, InternalPage, ID, "")
	'The properties are assigned to the global vars ensuring a property is not null.
	'Example getting a property: mKnobValue = Properties.Get("value")
	SetKnobProperties(Properties)
End Sub

' Runs when an object is created for the first time. Expects a valid html string which will get surrounded by a div automatically
' To handle multiple knobs, set the internal ID as selector
' The input selector can have as attributes additional knob property settings: example data-linecap=round 
' In addition div attributes can be set, i.e. style="width: 30%; border: 3px dashed; margin-bottom: 20px"
Sub ABMComp_Build(InternalPage As ABMPage, internalID As String) As String
	Dim htmlknob As String = $"<div id="${internalID}">
			 	<input id="${ABMComp.ID}i" Type="text" value=${mKnobValue} data-linecap=Round data-width="50%" class="${internalID}">
			 </div>"$
	Return htmlknob
End Sub

' Run the initalisation script creating the component. Global vars are used to set the properties.
' The change event is used to pass the value, via b4j_raiseEvent to the ABMComp_ID_changed event used in the CustomCanvasKnobPage.
' In addition a b4j_raiseEvent is used firsttime as well.
' HINT: Be careful pasting JavaScript code as the B4J IDE sets basic syntax which might cause JavaScript errors:
' (JS = Basic) min = Min, max = Max, if = If ...
Sub ABMComp_FirstRun(InternalPage As ABMPage, internalID As String)
	Log($"ABMComp_FirstRun: InternalID:${internalID}"$)
	Dim script As String = $"//Init the custom knob
		$('.${internalID}').knob({
			"min":${mKnobMin},
			"max":${mKnobMax},
			// "height":${mKnobHeight},
			// "width":${mKnobWidth},
			"fgColor":"${mKnobfgColor}",
			"skin":"tron",
			// "thickness":".2",
			"displayInput":${mKnobDisplayInput},
			'change': function (v) {
				    	b4j_raiseEvent('${ABMComp.ID}_changed', {'value':v});
			    	}
		}); 
    	b4j_raiseEvent('${ABMComp.ID}_changed', {'value':${mKnobValue}});
	"$
	InternalPage.ws.Eval(script, Array As Object(ABMComp.ID))
	' flush not needed, it's done in the refresh method in the lib
End Sub

' Runs when a refresh is called
Sub ABMComp_Refresh(InternalPage As ABMPage, internalID As String)
'NOT USED
End Sub

' do the stuff needed when the object is removed
Sub ABMComp_CleanUp(InternalPage As ABMPage, internalID As String)
' No action	
End Sub

'Assign the knob properties to global vars which are used for creating the knob in ABMComp FirstRun.
'A property has a default value and is override if in the properties map.
Private Sub SetKnobProperties(Properties As Map)
	Dim DoLog As Boolean = False
	If DoLog Then Log(Properties)
	mKnobValue = 0
	If Properties.Get("value") <> Null Then mKnobValue = Properties.Get("value")
	If DoLog Then Log(mKnobValue)
	mKnobfgColor = "#FF0000"
	If Properties.Get("fgcolor") <> Null Then mKnobfgColor = Properties.Get("fgcolor")
	If DoLog Then Log(mKnobfgColor)
	mKnobMin = 0
	If Properties.Get("min") <> Null Then mKnobMin = Properties.Get("min")
	If DoLog Then Log(mKnobMin)
	mKnobMax = 100
	If Properties.Get("max") <> Null Then mKnobMax = Properties.Get("max")
	If DoLog Then Log(mKnobMax)
	mKnobHeight = 100
	If Properties.Get("height") <> Null Then mKnobHeight = Properties.Get("height")
	If DoLog Then Log(mKnobHeight)
	mKnobWidth = 100
	If Properties.Get("width") <> Null Then mKnobWidth = Properties.Get("width")
	If DoLog Then Log(mKnobWidth)
	mKnobDisplayInput = True
	If Properties.Get("displayinput") <> Null Then mKnobDisplayInput = Properties.Get("displayinput")
	If DoLog Then Log(mKnobDisplayInput)
End Sub
