﻿Type=Class
Version=5.9
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
'Class module
Sub Class_Globals
	Private ws As WebSocket 'ignore
	' will hold our page information
	Public page As ABMPage
	' page theme
	Private theme As ABMTheme
	' to access the constants
	Private ABM As ABMaterial 'ignore	
	' name of the page, must be the same as the class name (case sensitive!)
	Public Name As String = "CustomCanvasKnobPage"  '<-------------------------------------------------------- IMPORTANT
	' will hold the unique browsers window id
	Private ABMPageId As String = ""
	' your own variables
	' Helpers with inital values
	Private myToastId As Int	'ignore
	'Define the volume & base value labels as global as being updated when turning the knob.
	Private labelVolumeValue As ABMLabel
	Private labelBassValue As ABMLabel
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize
	' build the local structure IMPORTANT!
	BuildPage		
End Sub

Private Sub WebSocket_Connected (WebSocket1 As WebSocket)	
	Log("Connected")
		
	ws = WebSocket1		
	
	ABMPageId = ABM.GetPageID(page, Name,ws)
	
	Dim session As HttpSession = ABM.GetSession(ws, ABMShared.SessionMaxInactiveIntervalSeconds)
	'----------------------START MODIFICATION 4.00-------------------------------
	If session.IsNew Then
		session.Invalidate
		ABMShared.NavigateToPage(ws, "", "./")
		Return
	End If
	'----------------------END MODIFICATION 4.00-------------------------------
	
	If ABMShared.NeedsAuthorization Then
		If session.GetAttribute2("IsAuthorized", "") = "" Then
			ABMShared.NavigateToPage(ws, ABMPageId, "../")
			Return
		End If
	End If		
	ABM.UpdateFromCache(Me, ABMShared.CachedPages, ABMPageId, ws)		
	If page.ComesFromPageCache Then
    	' when we have a page that is cached it doesn't matter if it comes or not from a new connection we serve the cached version.
		Log("Comes from cache")		
    	page.Refresh		
    	page.FinishedLoading		
	Else
    	If page.WebsocketReconnected Then
			Log("Websocket reconnected")
        	' when we have a client that doesn't have the page in cache and it's websocket reconnected and also it's session is new - basically when the client had internet problems and it's session (and also cache) expired before he reconnected so the user has content in the browser but we don't have any on the server. So we need to reload the page.
        	' when a client that doesn't have the page in cache and it's websocket reconnected but it's session is not new - when the client had internet problems and when he reconnected it's session was valid but he had no cache for this page we need to reload the page as the user browser has content, reconnected but we have no content in cache
        	ABMShared.NavigateToPage (ws, ABMPageId, "./" & page.PageHTMLName)
    	Else
        	' when the client did not reconnected it doesn't matter if the session was new or not because this is the websockets first connection so no dynamic content in the browser ... we are going to serve the dynamic content...
        	Log("Websocket first connection")
			page.Prepare
			ConnectPage			
    	End If
	End If
	Log(ABMPageId)		
End Sub

Private Sub WebSocket_Disconnected
	Log("Disconnected")
End Sub

Sub Page_ParseEvent(Params As Map)
	Dim eventName As String = Params.Get("eventname")
	Dim eventParams() As String = Regex.Split(",",Params.Get("eventparams"))
	
	If eventName = "beforeunload" Then
		Log("preparing for url refresh")
		ABM.RemoveMeFromCache(ABMShared.CachedPages, ABMPageId)
		Return
	End If
	Dim caller As Object = page.GetEventHandler(Me, eventName)
	If caller = Me Then
		If SubExists(Me, eventName) Then
			Params.Remove("eventname")
			Params.Remove("eventparams")
			' BEGIN NEW DRAGDROP
			If eventName = "page_dropped" Then
				page.ProcessDroppedEvent(Params)
			End If
			' END NEW DRAGDROP
			Select Case Params.Size
				Case 0
					CallSub(Me, eventName)
				Case 1
					CallSub2(Me, eventName, Params.Get(eventParams(0)))
				Case 2
					If Params.get(eventParams(0)) = "abmistable" Then
						Dim PassedTables As List = ABM.ProcessTablesFromTargetName(Params.get(eventParams(1)))
						CallSub2(Me, eventName, PassedTables)
					Else
						CallSub3(Me, eventName, Params.Get(eventParams(0)), Params.Get(eventParams(1)))
					End If
				Case Else
					' cannot be called directly, to many param
					CallSub2(Me, eventName, Params)
			End Select
		End If
	Else
		CallSubDelayed2(caller, "ParseEvent", Params) 'ignore
	End If
End Sub

public Sub BuildTheme()
	' start with the base theme defined in ABMShared
	theme.Initialize("pagetheme")
	theme.AddABMTheme(ABMShared.MyTheme)

	' add additional themes specific for this page
	theme.AddCellTheme("aligncenter")
	theme.Cell("aligncenter").Align = ABM.CELL_ALIGN_CENTER
End Sub

public Sub BuildPage()
	' initialize the theme
	BuildTheme
	
	' initialize this page using our theme
	page.InitializeWithTheme(Name, "/ws/" & ABMShared.AppName & "/" & Name, False, ABMShared.SessionMaxInactiveIntervalSeconds, theme)
	page.ShowLoader=True
	page.PageHTMLName = "index.html"
	page.PageTitle = "CustomCanvasKnobEx"
	page.PageDescription = "ABMaterial Custom Component Shiny Knob Example"
	page.PageKeywords = ""
	page.PageSiteMapPriority = ""
	page.PageSiteMapFrequency = ABM.SITEMAP_FREQ_YEARLY
		
	page.ShowConnectedIndicator = True
				
	' adding a navigation bar
	ABMShared.BuildNavigationBar(page, "CustomCanvasKnobEx","", "", "", "")
			
	' create the page grid
	' EXAMPLE page.AddRowsM(5,True,0,0, "").AddCells12MP(1,0,0,0,0,"")

	'R1 1x6 + 1x6 = [Label Volume][Label Bass]
	page.AddRows(1,True,"").AddCellsOS(1,0,0,0,6,6,6,"").AddCellsOS(1,0,0,0,6,6,6,"")
	'R2 1x6 + 1x6 = [CustomKnob Volume][CustomKnob Bass]
	page.AddRows(1,True,"").AddCellsOS(1,0,0,0,6,6,6,"").AddCellsOS(1,0,0,0,6,6,6,"")
	'R3 1x6 + 1x6 = [Label VolumeValue][Label BassValue]
	page.AddRows(1,True,"").AddCellsOS(1,0,0,0,6,6,6,"").AddCellsOS(1,0,0,0,6,6,6,"")
	'R4 1x6 + 1x6 = [Button VolumeValue][Button BassValue]
	page.AddRows(1,True,"").AddCellsOS(1,0,0,0,6,6,6,"").AddCellsOS(1,0,0,0,6,6,6,"")

	'IMPORTANT to load the complete grid AND before you start adding components
	page.BuildGrid 

	' Extra javascript files and CSSfiles, to be added in the BuildPage() method!
	page.AddExtraJavaScriptFile("custom/jquery.knob.min.js")

	' Add more like BuildFooter		
	ABMShared.BuildFooter(page)
End Sub

public Sub ConnectPage()
	'	connecting the navigation bar
	ABMShared.ConnectNavigationBar(page)

	' Add the components
	' R1
	Dim labelVolume As ABMLabel
	labelVolume.Initialize(page,"labelVolume","Volume", ABM.SIZE_H5, True, "aligncenter")
	page.Cell(1,1).AddComponent(labelVolume)
	page.Cell(1,1).SetFixedHeight(50, False)
	Dim labelBass As ABMLabel
	labelBass.Initialize(page,"labelBass","Bass", ABM.SIZE_H5, True, "aligncenter")
	page.Cell(1,2).AddComponent(labelBass)
	page.Cell(1,2).SetFixedHeight(50, False)

	' R2
	Dim custCanvasKnobVolume As CustomCanvasKnob
	Dim m As Map = CreateMap("value":33)
	custCanvasKnobVolume.Initialize(page, "custCanvasKnobVolume", m)
	page.Cell(2,1).AddComponent(custCanvasKnobVolume.ABMComp)
'	page.Cell(2,1).SetFixedHeight(50, False)
	Dim custCanvasKnobBass As CustomCanvasKnob
	Dim m As Map = CreateMap("value":69, "fgcolor":"#0000FF")
	custCanvasKnobBass.Initialize(page, "custCanvasKnobBass", m)
	page.Cell(2,2).AddComponent(custCanvasKnobBass.ABMComp)
'	page.Cell(2,2).SetFixedHeight(50, False)

	' R3
	labelVolumeValue.Initialize(page,"labelVolumeValue","0", ABM.SIZE_PARAGRAPH, True, "aligncenter")
	page.Cell(3,1).AddComponent(labelVolumeValue)
'	page.Cell(3,1).SetFixedHeight(50, False)
	labelBassValue.Initialize(page,"labelBassValue","0", ABM.SIZE_PARAGRAPH, True, "aligncenter")
	page.Cell(3,2).AddComponent(labelBassValue)
'	page.Cell(3,2).SetFixedHeight(50, False)

	' refresh the page
	page.Refresh
	
	' Tell the browser we finished loading
	page.FinishedLoading

	' restoring the navigation bar position
	page.RestoreNavigationBarPosition

End Sub

Sub page_MsgboxResult(returnName As String, result As String)
	
End Sub

Sub Page_Dropped(Params As Map)
	If Params.Get("before") = "" Then
		Log("'" & Params.Get("component") & "' moved from " & Params.Get("source") & " to " & Params.Get("target"))
	Else
		Log("'" & Params.Get("component") & "' moved from " & Params.Get("source") & " to " & Params.Get("target") & " before component " & Params.Get("before"))
	End If	
End Sub

' clicked on the navigation bar
Sub Page_NavigationbarClicked(Action As String, Value As String)
	' saving the navigation bar position
	page.SaveNavigationBarPosition
	If Action = "LogOff" Then
		ABMShared.LogOff(page)
		Return
	End If

	ABMShared.NavigateToPage(ws, ABMPageId, Value)
End Sub

Sub Page_InputboxResult(returnName As String, result As String)
	
End Sub

Sub Page_DebugConsole(message As String)
	Log("---> " & message)
End Sub

Sub Page_FileUploaded(FileName As String, success As Boolean)
	
End Sub

Sub Page_ToastClicked(toastID As String, Action As String)
		
End Sub

Sub Page_ToastDismissed(ToastId As String)
	
End Sub

Sub Page_Authenticated(Params As Map)
	
End Sub

Sub Page_FirebaseAuthError(extra As String)
	
End Sub

Sub Page_FirebaseAuthStateChanged(IsLoggedIn As Boolean)
	
End Sub

Sub Page_FirebaseStorageError(jobID As String, extra As String)
	
End Sub

Sub Page_FirebaseStorageResult(jobID As String, extra As String)
	
End Sub

Sub Page_ModalSheetDismissed(ModalSheetName As String)
	
End Sub

Sub Page_NextContent(TriggerComponent As String)
	
End Sub

Sub Page_SignedOffSocialNetwork(Network As String, Extra As String)
	
End Sub

Sub page_CellClicked(Target As String, RowCell As String)
	
End Sub

Sub page_RowClicked(Target As String, Row As String)
	
End Sub

Sub Page_NativeResponse(jsonMessage As String)
	
End Sub

Sub Page_NavigationbarSearch(search As String)
	
End Sub

#Region CustomKnob
'Handle changes of the custom knob value. Update the label Volume Value
Sub custCanvasKnobVolume_Changed(Params As Map)
	Log($"custCanvasKnobVolume_Changed=${Params.Get("value")}"$)
	If Not(Params.IsInitialized) Then Return
	labelVolumeValue.Text = NumberFormat(Params.Get("value"),0,0)
	page.Refresh
End Sub

'Handle changes of the custom knob base. Update the label Bass Value
Sub custCanvasKnobBass_Changed(Params As Map)
	Log($"custCanvasKnobBase_Changed=${Params.Get("value")}"$)
	If Not(Params.IsInitialized) Then Return
	labelBassValue.Text = NumberFormat(Params.Get("value"),0,0)
	page.Refresh
End Sub
#End Region
